@@ -1,6 +1,6 @@
 from operator import itemgetter
-from collections import Counter
-import copy
+from collections import defaultdict
+from random import Random
 
 
 class OperationOrLanguage(object):
@@ -26,7 +26,7 @@ class Operation(OperationOrLanguage):
         """
         self.varstacks = tuple(map(itemgetter(0), varstacks))
         self.name = name or self.__class__.__name__.lower()
-        self.requirements = Counter()
+        self.requirements = defaultdict(lambda: 0)
         for s, c in varstacks:
             self.requirements[s] += c
         self.pattern = pattern
@@ -64,7 +64,7 @@ class ReadAndWrite(Operation):
         pattern=None
     ):
         super(ReadAndWrite, self).__init__(
-            Counter(argspec).items(),
+            _counts(argspec),
             name or function.__name__,
             pattern=pattern,
             precondition=precondition,
@@ -86,7 +86,7 @@ class Mutate(Operation):
         pattern=None
     ):
         super(ReadAndWrite, self).__init__(
-            Counter(argspec).items(),
+            _counts(argspec),
             name or function.__name__,
             pattern=pattern,
             precondition=precondition,
@@ -136,9 +136,12 @@ class UnaryOperator(ReadAndWrite):
 class Check(Operation):
     def __init__(self, test, argspec, name=None, pattern=None):
         name = name or test.__name__
-        pattern = pattern or ("assert %s(%%s)" % (name,))
+        if not pattern:
+            arg_string = ', '.join(["%s"] * len(argspec))
+            pattern = "assert %s(%s)" % (name, arg_string)
+
         super(Check, self).__init__(
-            Counter(argspec).items(), name=name, pattern=pattern
+            _counts(argspec), name=name, pattern=pattern
         )
         self.argspec = argspec
         self.test = test
@@ -166,20 +169,20 @@ class SingleStackOperation(Operation):
 
 
 class Push(SingleStackOperation):
-    def __init__(self, varstack, value):
+    def __init__(self, varstack, gen_value):
         super(Push, self).__init__(varstack, name="push")
-        self.value = value
+        self.gen_value = gen_value
 
     def compile(self, arguments, results):
         assert not arguments
         assert len(results) == 1
-        return ["%s = %r" % (results[0], self.value)]
+        return ["%s = %r" % (results[0], self.gen_value())]
 
     def invoke(self, context):
-        context.varstack(self.varstack).push(copy.deepcopy(self.value))
+        context.varstack(self.varstack).push(self.gen_value())
 
     def args(self):
-        return super(Push, self).args() + (self.value,)
+        return super(Push, self).args() + (self.gen_value(),)
 
 
 class InapplicableLanguage(Exception):
@@ -209,9 +212,15 @@ class PushRandom(Language):
         self.target = target
 
     def generate(self, context):
+        state = context.random.getstate()
+
+        def gen_result():
+            r = Random(0)
+            r.setstate(state)
+            return self.produce(r)
         return Push(
             self.target,
-            self.produce(context.random)
+            gen_result,
         )
 
 
@@ -219,7 +228,7 @@ class ChooseFrom(Language):
     def __init__(self, children):
         super(ChooseFrom, self).__init__()
         self.children = tuple(children)
-        self.requirements = Counter()
+        self.requirements = defaultdict(lambda: 0)
         for c in children:
             for k, v in c.requirements.items():
                 self.requirements[k] = min(v, self.requirements[k])
@@ -287,3 +296,10 @@ class Rot(SingleStackOperation):
 
     def compile(self, arguments, results):
         return []
+
+
+def _counts(values):
+    c = defaultdict(lambda: 0)
+    for v in values:
+        c[v] += 1
+    return c.items()
