@@ -1,3 +1,4 @@
+from __future__ import print_function
 from random import Random
 from contextlib import contextmanager
 from .operations import (
@@ -8,7 +9,7 @@ from .operations import (
     BinaryOperator,
     UnaryOperator,
 )
-from collections import namedtuple, Counter
+from collections import namedtuple, defaultdict
 import traceback
 
 
@@ -147,7 +148,7 @@ class RunContext(object):
 
     def read(self, argspec):
         result = []
-        seen = Counter()
+        seen = defaultdict(lambda: 0)
         for a in argspec:
             result.append(self.varstack(a).peek(seen[a]))
             seen[a] += 1
@@ -171,7 +172,7 @@ class TestMachine(object):
     def inform(self, message, *args, **kwargs):
         assert not (args and kwargs)
         if self.print_output:
-            print (message % (args or kwargs))
+            print(message % (args or kwargs))
 
     def operation(self, *args, **kwargs):
         """
@@ -192,7 +193,7 @@ class TestMachine(object):
         """
         self.add_language(Check(*args, **kwargs))
 
-    def generate(self, produce, target, name=None):
+    def generate(self, produce, target, name=None, supports_copy=True):
         """
         Add a generator for operations which produces values by calling
         produce with a Random instance and pushes them onto target.
@@ -251,7 +252,7 @@ class TestMachine(object):
         examples_found = 0
         best_example = None
 
-        for _ in xrange(self.n_iters):
+        for _ in range(self.n_iters):
             program = []
             context = RunContext()
             for _ in xrange(self.prog_length):
